

#include <stdint.h>
#include <string.h>
#include <stdarg.h>
#include "Lcd20x4Screen.H"


using namespace avelino;
using namespace std;


CharacterBitmapEntry::CharacterBitmapEntry(char latin1Char, char lcdFallbackChar, uint8_t row0, uint8_t row1, uint8_t row2, uint8_t row3, uint8_t row4, uint8_t row5, uint8_t row6, uint8_t row7) {
	this->latin1Char = latin1Char;
	this->lcdChar = 0;
	this->lcdFallbackChar = lcdFallbackChar;
	this->timesUsed = 0;
	this->loadMark = false;
	this->bitmap[0] = row0;
	this->bitmap[1] = row1;
	this->bitmap[2] = row2;
	this->bitmap[3] = row3;
	this->bitmap[4] = row4;
	this->bitmap[5] = row5;
	this->bitmap[6] = row6;
	this->bitmap[7] = row7;
}


CharacterBitmapEntry *Lcd20x4Screen::findCharacterBitmapEntry(char c) {
	uint8_t n;
	CharacterBitmapEntry *p = this->getCharacterBitmapEntryTable(n);
	for (uint8_t i = 0; i < n; i++, p++) {
		if (p->latin1Char == c)
			return p;
	}
	return NULL;
}


void Lcd20x4Screen::init() {
	Lcd20x4::init();
	memset(this->logicalDisplay, 0, 80);
	memset(this->physicalDisplay, 0, 80);
	this->availableLcdChars.push(1);
	this->availableLcdChars.push(2);
	this->availableLcdChars.push(3);
	this->availableLcdChars.push(4);
	this->availableLcdChars.push(5);
	this->availableLcdChars.push(6);
	this->availableLcdChars.push(7);
}


uint8_t Lcd20x4Screen::replaceChar(char newC, char oldC) {
	CharacterBitmapEntry *e = findCharacterBitmapEntry(oldC);
	if (e != NULL) {
		e->timesUsed--;
		if (e->timesUsed == 0) {
			this->availableLcdChars.push(e->lcdChar);
			e->lcdChar = 0;
		}
	}
	e = findCharacterBitmapEntry(newC);
	if (e == NULL)
		return newC;	// the special character is not mapped
	else {
		if (e->lcdChar > 0) {
			// the special character is mapped and already used
			e->timesUsed++;
		}
		else {
			// the special character is mapped but not used
			if (this->availableLcdChars.isEmpty()) {
				// there are no free entries
				// return fallback character
				return e->lcdFallbackChar;
			}
			e->lcdChar = this->availableLcdChars.pop();
			e->timesUsed = 1;
			e->loadMark = true;
			//this->loadBitmap(e->lcdChar, e->bitmap);
		}
		return e->lcdChar;
	}
}


void Lcd20x4Screen::loadMarkedCharacterBitmapEntries() {
	uint8_t n;
	CharacterBitmapEntry *p = this->getCharacterBitmapEntryTable(n);
	for (uint8_t i = 0; i < n; i++, p++) {
		if (p->loadMark) {
			this->loadBitmap(p->lcdChar, p->bitmap);
			p->loadMark = false;
		}
	}
}


void Lcd20x4Screen::run() {
	Lcd20x4::run();
	for (uint8_t y = 0; y < 4; y++) {
		char *logical = this->logicalDisplay[y];
		char *l = logical;
		char *physical = this->physicalDisplay[y];
		char *p = physical;
		uint8_t x, x1;
		bool insideChange = false;
		for (x = 0; x < 20; x++) {
			if (!insideChange) {
				if (*l != *p) {
					x1 = x;
					insideChange = true;
				}
			}
			else {
				if (*l == *p) {
					this->queueLocate(x1, y);
					this->printWithReplacement(logical, physical, x1, x);
					insideChange = false;
				}
			}
			*p = *l;
			l++;
			p++;
		}
		if (insideChange) {
			this->queueLocate(x1, y);
			this->printWithReplacement(logical, physical, x1, x);
		}
	}
	this->loadMarkedCharacterBitmapEntries();
}


void Lcd20x4Screen::printWithReplacement(char *logicalRow, char *physicalRow, uint8_t x, uint8_t xOut) {
	while (x < xOut) {
		uint8_t v = this->replaceChar(logicalRow[x], physicalRow[x]);
		this->queueSingleData(v);
		x++;
	}
}


void Lcd20x4Screen::print(uint8_t x, uint8_t y, const char *s) {
	memcpy(&this->logicalDisplay[y][x], s, strlen(s));
}


char Lcd20x4Screen::HEX_TABLE[16] = {
	'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'
};


void Lcd20x4Screen::printHex(uint8_t x, uint8_t y, uint8_t v) {
	this->logicalDisplay[y][x] = HEX_TABLE[v >> 4];
	this->logicalDisplay[y][x + 1] = HEX_TABLE[v & 0x0F];
}


void Lcd20x4Screen::printDec(uint8_t x, uint8_t y, uint16_t v) {
	uint8_t stack[8];
	int8_t stackTop = 0;
	if (v == 0)
		stack[stackTop++] = 0;
	while (v > 0) {
		stack[stackTop++] = v % 10;
		v = v / 10;
	}
	while (stackTop > 0) {
		stackTop--;
		this->logicalDisplay[y][x] = '0' + stack[stackTop];
		x++;
	}
}

