/*
 * vim: ts=4 nobackup
 */


#include <avr/io.h>
#include <avr/interrupt.h>
#include "Timer.H"


using namespace avelino;
using namespace std;


int16_t Timer::counter[NUM_TIMER_COUNTERS];


void Timer::init() {
	for (uint8_t i = 0; i < NUM_TIMER_COUNTERS; i++)
		counter[i] = -1;
	cli();
	// PRR1 |= 1 << PRUSB;
	TCCR1A = 0;
	TCCR1B = 0;
	// final Hz = F_CPU / (2 * prescaler * (1 + OCR1A))
	OCR1A = 624;   // 200 Hz = 5ms with prescaler = 64
	// configure timer for CTC mode and start timer 1 at Fcpu/64 = 16000000 / 64 = 250000 Hz
	TCNT1 = 0;
	TCCR1B |= ((1 << WGM12) | (1 << CS10) | (1 << CS11));
	TIMSK1 = (1 << OCIE1A);
	sei();
}


int8_t Timer::alloc() {
	for (uint8_t i = 0; i < NUM_TIMER_COUNTERS; i++) {
		if (counter[i] == -1) {
			counter[i] = 0;
			return i;
		}
	}
	return -1;
}


void Timer::__isr() {
	for (uint8_t i = 0; i < NUM_TIMER_COUNTERS; i++) {
		if (counter[i] > 0)
			counter[i]--;
	}
}


void Timer::free(int8_t handle) {
	counter[handle] = -1;
}


ISR(TIMER1_COMPA_vect) {
	Timer::__isr();
}


