/*
 * vim: ts=4
 */


#ifndef  __USB_H__
#define  __USB_H__


#include <stdint.h>


extern "C++" {
	namespace avelino {
		using namespace std;

		class USBSetupPacket {
			public:
				static const uint8_t REQUEST_GET_DESCRIPTOR;
				static const uint8_t REQUEST_SET_ADDRESS;
				static const uint8_t REQUEST_GET_STATUS;
				static const uint8_t REQUEST_GET_CONFIGURATION;
				static const uint8_t REQUEST_SET_CONFIGURATION;
				uint8_t bmRequestType;
				uint8_t bRequest;
				uint16_t wValue;
				uint16_t wIndex;
				uint16_t wLength;
		} __attribute__ ((packed));

		class USB {
			private:
				static const uint8_t STATUS_IDLE;
				static const uint8_t STATUS_HALT;
				static const uint8_t DEVICE_DESCRIPTOR;
				static const uint8_t CONFIGURATION_DESCRIPTOR;
				static uint8_t whichDescriptor;
				static uint8_t buffer[64];
				static uint8_t *toSend;
				static uint16_t toSendSize;
				static void readOnBuffer(uint16_t bytes, uint8_t *buffer);
				static void writeFromBuffer(uint16_t bytes, uint8_t *buffer);
				static bool configureEndpoint0();
				static void sendDataToEndpoint0();
			public:
				static uint8_t rxBuffer[64];
				static bool rxBufferFull;
				static uint8_t txBuffer[64];
				static uint8_t status;
				static USBSetupPacket *setupPacketReceived;
				static uint8_t address;
				static void powerOff();
				static void init();
				static void done();
				static void __general_isr();
				static void __endpoint_isr();
		};
	}
}


#endif  // __USB_H__
