

#ifndef  __USBDESCRIPTOR_H__
#define  __USBDESCRIPTOR_H__


#include <stdint.h>


extern "C++" {
	namespace avelino {
		using namespace std;

		class USBDeviceDescriptor {
			public:
				uint8_t bLength;
				uint8_t bDescriptorType;
				uint16_t bcdUSB;
				uint8_t bDeviceClass;
				uint8_t bDeviceSubClass;
				uint8_t bDeviceProtocol;
				uint8_t bMaxPacketSize0;
				uint16_t idVendor;
				uint16_t idProduct;
				uint16_t bcdDevice;
				uint8_t iManufacturer;
				uint8_t iProduct;
				uint8_t iSerialNumber;
				uint8_t bNumConfigurations;
		} __attribute__ ((packed));

		class USBConfigurationDescriptor {
			public:
				uint8_t bLength;
				uint8_t bDescriptorType;
				uint16_t wTotalLength;
				uint8_t bNumInterfaces;
				uint8_t bConfigurationValue;
				uint8_t iConfiguration;
				uint8_t bmAttributes;
				uint8_t maxPower;
		} __attribute__ ((packed));

		class USBInterfaceDescriptor {
			public:
				uint8_t bLength;
				uint8_t bDescriptorType;
				uint8_t bInterfaceNumber;
				uint8_t bAlternateSetting;
				uint8_t bNumEndpoints;
				uint8_t bInterfaceClass;
				uint8_t bInterfaceSubClass;
				uint8_t bInterfaceProtocol;
				uint8_t iInterface;
		} __attribute__ ((packed));

		class USBHIDDescriptor {
			public:
				uint8_t bLength;
				uint8_t bDescriptorType;
				uint16_t bcdHID;
				uint8_t bCountryCode;
				uint8_t bNumDescriptors;
				uint8_t bReportDescriptorType;
				uint8_t wDescriptorLength;
		} __attribute__ ((packed));

		class USBEndpointDescriptor {
			public:
				uint8_t bLength;
				uint8_t bDescriptorType;
				uint8_t bEndpointAddress;
				uint8_t bmAttributes;
				uint16_t wMaxPacketSize;
				uint8_t bInterval;
		} __attribute__ ((packed));

		#define  USB_HID_REPORT_DESCRIPTOR_SIZE  63

		class USBHIDReportDescriptor {
			public:
				uint8_t data[USB_HID_REPORT_DESCRIPTOR_SIZE];
		} __attribute__ ((packed));

		class USBHIDDescriptorContainer {
			public:
				USBDeviceDescriptor deviceDescriptor;
				USBConfigurationDescriptor configurationDescriptor;
				USBInterfaceDescriptor interfaceDescriptor;
				USBHIDDescriptor hidDescriptor;
				USBEndpointDescriptor endpointDescriptor;
				USBHIDReportDescriptor hidReportDescriptor;
		} __attribute__ ((packed));

		const USBHIDDescriptorContainer MyUSBHIDDescriptorContainer = {
			{  // device descriptor
				0x12,
				0x01,
				0x0100,
				0x00,
				0x00,
				0x00,
				0x40,    // bMaxPacketSize0 = 64  (for endpoint 0)
				0x1234,  // idVendor
				0x5678,  // idProduct
				0x0100,
				0x00,
				0x00,
				0x00,
				0x01     // one available configuration
			},
			{  // configuration descriptor
				0x09,
				0x02,
				0x003B,
				0x02,    // two interfaces
				0x01,
				0x00,
				0x00,    // not self powered
				0x32
			},
			{  // interface descriptor
				0x09,
				0x04,
				0x00,
				0x00,
				0x01,
				0x03,
				0x01,    // boot interface
				0x01,    // boot keyboard
				0x00
			},
			{  // hid descriptor
				0x09,
				0x21,
				0x0101,
				0x00,
				0x01,
				0x22,
				USB_HID_REPORT_DESCRIPTOR_SIZE    // hid report descriptor size
			},
			{  // endpoint descriptor
				0x07,
				0x05,
				0x81,     // in endpoint 1
				0x03,     // interrupt endpoint
				0x0040,   // max packet size = 64
				0x0A      // 10ms for interval polling
			},
			/*
			{  // hid report descriptor form a simple keypad (generated using HIDTool from USB.org)
				{
					0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
					0x09, 0x06,                    // USAGE (Keyboard)
					0xa1, 0x01,                    // COLLECTION (Application)
					0x95, 0x01,                    //   REPORT_COUNT (1)
					0x75, 0x08,                    //   REPORT_SIZE (8)
					0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
					0x25, 0x65,                    //   LOGICAL_MAXIMUM (101)
					0x05, 0x07,                    //   USAGE_PAGE (Keyboard)
					0x19, 0x00,                    //   USAGE_MINIMUM (Reserved (no event indicated))
					0x29, 0x65,                    //   USAGE_MAXIMUM (Keyboard Application)
					0x81, 0x02,                    //   INPUT (Data,Var,Abs)
					0xc0                           // END_COLLECTION
				}
			}
			*/
			{  // HID report descriptor for a default boot keyboard
				0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
				0x09, 0x06,                    // USAGE (Keyboard)
				0xa1, 0x01,                    // COLLECTION (Application)
				0x05, 0x07,                    //   USAGE_PAGE (Keyboard)
				0x19, 0xe0,                    //   USAGE_MINIMUM (Keyboard LeftControl)
				0x29, 0xe7,                    //   USAGE_MAXIMUM (Keyboard Right GUI)
				0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
				0x25, 0x01,                    //   LOGICAL_MAXIMUM (1)
				0x75, 0x01,                    //   REPORT_SIZE (1)
				0x95, 0x08,                    //   REPORT_COUNT (8)
				0x81, 0x02,                    //   INPUT (Data,Var,Abs)
				0x95, 0x01,                    //   REPORT_COUNT (1)
				0x75, 0x08,                    //   REPORT_SIZE (8)
				0x81, 0x03,                    //   INPUT (Cnst,Var,Abs)
				0x95, 0x05,                    //   REPORT_COUNT (5)
				0x75, 0x01,                    //   REPORT_SIZE (1)
				0x05, 0x08,                    //   USAGE_PAGE (LEDs)
				0x19, 0x01,                    //   USAGE_MINIMUM (Num Lock)
				0x29, 0x05,                    //   USAGE_MAXIMUM (Kana)
				0x91, 0x02,                    //   OUTPUT (Data,Var,Abs)
				0x95, 0x01,                    //   REPORT_COUNT (1)
				0x75, 0x03,                    //   REPORT_SIZE (3)
				0x91, 0x03,                    //   OUTPUT (Cnst,Var,Abs)
				0x95, 0x06,                    //   REPORT_COUNT (6)
				0x75, 0x08,                    //   REPORT_SIZE (8)
				0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
				0x25, 0x65,                    //   LOGICAL_MAXIMUM (101)
				0x05, 0x07,                    //   USAGE_PAGE (Keyboard)
				0x19, 0x00,                    //   USAGE_MINIMUM (Reserved (no event indicated))
				0x29, 0x65,                    //   USAGE_MAXIMUM (Keyboard Application)
				0x81, 0x00,                    //   INPUT (Data,Ary,Abs)
				0xc0                           // END_COLLECTION

			}
		};

		class USBSerialDescriptorContainer {
			public:
				USBDeviceDescriptor deviceDescriptor;
				USBConfigurationDescriptor configurationDescriptor;
				USBInterfaceDescriptor interfaceDescriptor;
				USBEndpointDescriptor inEndpointDescriptor;
				USBEndpointDescriptor outEndpointDescriptor;
		} __attribute__ ((packed));

		const USBSerialDescriptorContainer MyUSBSerialDescriptorContainer = {
			{  // device descriptor
				0x12,
				0x01,
				0x0100,
				0x00,
				0x00,
				0x00,
				0x40,    // bMaxPacketSize0 = 64  (for endpoint 0)
				0xA4F6,  // idVendor
				0x5678,  // idProduct
				0x0100,
				0x00,
				0x00,
				0x00,
				0x01     // one available configuration
			},
			{  // configuration descriptor
				0x09,
				0x02,
				0x0020,  // total size of this descriptor and rest of descriptors inside this configuration 9 + 9 + 7 + 7 = 32 bytes
				0x01,
				0x01,
				0x00,
				0x80,    // not self powered
				0x04     // max power in units of 5 mA
			},
			{  // interface descriptor
				0x09,
				0x04,
				0x00,
				0x00,
				0x02,
				0xFF,   // class = vendor specific
				0x00,   // subclass
				0x00,
				0x00
			},
			{  // in endpoint descriptor
				0x07,
				0x05,
				0x81,     // in endpoint 1
				0x02,     // bulk endpoint
				0x0040,   // max packet size = 64
				0x0A      // 10ms for interval polling
			},
			{  // out endpoint descriptor
				0x07,
				0x05,
				0x02,     // out endpoint 2
				0x02,     // bulk endpoint
				0x0040,   // max packet size = 64
				0x0A      // 10ms for interval polling
			}
		};
	}
}


#endif  // __USBDESCRIPTOR_H__
